class Appfirst::Logger < Faraday::Response::Middleware
  extend Forwardable

  def initialize(app, logger = nil)
    super(app)
    @logger = logger || ::Logger.new(nil)
  end

  def_delegators :@logger, :debug, :info, :warn, :error, :fatal

  def call(env)
    info "REQUEST: #{env[:method].upcase} #{env[:url].to_s}"
    info('request') { dump_headers env[:request_headers] }
    info('request.body') { env[:body] } if (env[:request_headers]["Accept"] || "").match("application/json")
    info('')
    super
  end

  def on_complete(env)
    info "RESPONSE: #{env[:status]}"
    info('response') { dump_headers env[:response_headers] }
    info('response.body') { env[:body] } if (env[:response_headers]["Content-Type"] || "").match("application/json")
    info('')
  end

  private

  def dump_headers(headers)
    headers.map { |k, v| "#{k}: #{v.inspect}" }.join("\n")
  end
end
